import log from "loglevel";
interface IPrincipal {
    id: string;
    policyVersion?: unknown;
    roles: string[];
    attr?: {
        [key: string]: unknown;
    };
}
interface IAuxData {
    jwt?: IJwt;
}
interface IJwt {
    token: string;
    keySetId?: string;
}
interface IAuthorize {
    actions: string[];
    resource: {
        policyVersion?: unknown;
        kind: string;
        instances: {
            [resourceKey: string]: {
                attr?: {
                    [key: string]: unknown;
                };
            };
        };
    };
    principal: IPrincipal;
    auxData?: IAuxData;
}
export declare enum ValidationErrorSource {
    SOURCE_RESOURCE = "SOURCE_RESOURCE",
    SOURCE_PRINCIPAL = "SOURCE_PRINCIPAL"
}
export declare enum AuthorizeEffect {
    ALLOW = "EFFECT_ALLOW",
    DENY = "EFFECT_DENY"
}
export declare class AuthorizationError extends Error {
}
export interface ICerbosResponse {
    isAuthorized: (resourceKey: string, action: string) => boolean;
}
interface CerbosOptions {
    hostname: string;
    logLevel?: log.LogLevelDesc;
    handleValidationErrors?: "error" | "log" | false;
    playgroundInstance?: string;
}
export declare class Cerbos {
    private host;
    private log;
    private playgroundInstance?;
    private handleValidationErrors?;
    constructor({ hostname, logLevel, playgroundInstance, handleValidationErrors, }: CerbosOptions);
    check(data: IAuthorize): Promise<ICerbosResponse>;
}
export {};
