"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Cerbos = exports.AuthorizationError = exports.AuthorizeEffect = exports.ValidationErrorSource = void 0;
const isomorphic_unfetch_1 = __importDefault(require("isomorphic-unfetch"));
const uuid_1 = require("uuid");
const loglevel_1 = __importDefault(require("loglevel"));
var ValidationErrorSource;
(function (ValidationErrorSource) {
    ValidationErrorSource["SOURCE_RESOURCE"] = "SOURCE_RESOURCE";
    ValidationErrorSource["SOURCE_PRINCIPAL"] = "SOURCE_PRINCIPAL";
})(ValidationErrorSource = exports.ValidationErrorSource || (exports.ValidationErrorSource = {}));
// export interface ICerbosBatchAuthorizeResource {
//   actions: string[];
//   resource: {
//     policyVersion?: unknown;
//     kind: string;
//     id: string;
//     attr: {
//       [key: string]: unknown;
//     };
//   };
// }
// export interface ICerbosBatchAuthorizeResult {
//   [key: string]: AuthorizeEffect;
// }
// interface IBatchAuthorize {
//   principal: IPrincipal;
//   resources: ICerbosBatchAuthorizeResource[];
// }
// interface IAuthorizeBatchResponse {
//   requestID: string;
//   results: {
//     resourceId: string;
//     actions: ICerbosBatchAuthorizeResult;
//   }[];
// }
// interface ICerbosBatchResponse {
//   resourceId: string;
//   actions: {
//     [action: string]: AuthorizeEffect;
//   };
// }
var AuthorizeEffect;
(function (AuthorizeEffect) {
    AuthorizeEffect["ALLOW"] = "EFFECT_ALLOW";
    AuthorizeEffect["DENY"] = "EFFECT_DENY";
})(AuthorizeEffect = exports.AuthorizeEffect || (exports.AuthorizeEffect = {}));
class AuthorizationError extends Error {
}
exports.AuthorizationError = AuthorizationError;
class CerbosResponseWrapper {
    constructor(resp) {
        this.resp = resp;
    }
    isAuthorized(resourceKey, action) {
        var _a;
        try {
            const allowed = ((_a = this.resp.resourceInstances[resourceKey]) === null || _a === void 0 ? void 0 : _a.actions[action]) ==
                AuthorizeEffect.ALLOW;
            return allowed !== null && allowed !== void 0 ? allowed : false;
        }
        catch (e) {
            return false;
        }
    }
}
class Cerbos {
    constructor({ hostname, logLevel, playgroundInstance, handleValidationErrors, }) {
        this.host = hostname;
        this.playgroundInstance = playgroundInstance;
        this.handleValidationErrors = handleValidationErrors;
        this.log = loglevel_1.default.noConflict();
        this.log.setLevel(logLevel !== null && logLevel !== void 0 ? logLevel : "silent");
    }
    check(data) {
        return __awaiter(this, void 0, void 0, function* () {
            this.log.info("Cerbos.check", data);
            const payload = Object.assign(Object.assign({ requestId: (0, uuid_1.v4)() }, data), { resource: Object.assign({ policyVersion: data.resource.policyVersion || "default" }, data.resource), principal: Object.assign({ policyVersion: data.principal.policyVersion || "default" }, data.principal) });
            this.log.debug("Cerbos.check Payload", payload);
            let headers = {
                "Content-Type": "application/json",
            };
            if (this.playgroundInstance) {
                headers = Object.assign(Object.assign({}, headers), { "Playground-Instance": this.playgroundInstance });
            }
            let resp;
            // Fetch Data
            try {
                const response = yield (0, isomorphic_unfetch_1.default)(`${this.host}/api/check`, {
                    method: "post",
                    body: JSON.stringify(payload),
                    headers,
                });
                resp = yield response.json();
                this.log.info("Cerbos.check: Response", resp);
            }
            catch (e) {
                this.log.error("Cerbos.check Error", e);
                throw new AuthorizationError(`Could not connect to Cerbos PDP at ${this.host}`);
            }
            // Handle Validation Errors
            if (this.handleValidationErrors) {
                const validationErrors = resp.resourceInstances
                    ? Object.values(resp.resourceInstances)
                        .map((resource) => resource.validationErrors)
                        .flat()
                    : [];
                if (validationErrors.length > 0) {
                    if (this.handleValidationErrors === "error") {
                        throw new AuthorizationError(`Validation Error: ${JSON.stringify(validationErrors)}`);
                    }
                    else {
                        this.log.error("Cerbos.check: Validation Errors", validationErrors);
                    }
                }
            }
            return new CerbosResponseWrapper(resp);
        });
    }
}
exports.Cerbos = Cerbos;
