var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import fetch from "isomorphic-unfetch";
import { v4 as uuidv4 } from "uuid";
import log from "loglevel";
export var ValidationErrorSource;
(function (ValidationErrorSource) {
    ValidationErrorSource["SOURCE_RESOURCE"] = "SOURCE_RESOURCE";
    ValidationErrorSource["SOURCE_PRINCIPAL"] = "SOURCE_PRINCIPAL";
})(ValidationErrorSource || (ValidationErrorSource = {}));
// export interface ICerbosBatchAuthorizeResource {
//   actions: string[];
//   resource: {
//     policyVersion?: unknown;
//     kind: string;
//     id: string;
//     attr: {
//       [key: string]: unknown;
//     };
//   };
// }
// export interface ICerbosBatchAuthorizeResult {
//   [key: string]: AuthorizeEffect;
// }
// interface IBatchAuthorize {
//   principal: IPrincipal;
//   resources: ICerbosBatchAuthorizeResource[];
// }
// interface IAuthorizeBatchResponse {
//   requestID: string;
//   results: {
//     resourceId: string;
//     actions: ICerbosBatchAuthorizeResult;
//   }[];
// }
// interface ICerbosBatchResponse {
//   resourceId: string;
//   actions: {
//     [action: string]: AuthorizeEffect;
//   };
// }
export var AuthorizeEffect;
(function (AuthorizeEffect) {
    AuthorizeEffect["ALLOW"] = "EFFECT_ALLOW";
    AuthorizeEffect["DENY"] = "EFFECT_DENY";
})(AuthorizeEffect || (AuthorizeEffect = {}));
export class AuthorizationError extends Error {
}
class CerbosResponseWrapper {
    constructor(resp) {
        this.resp = resp;
    }
    isAuthorized(resourceKey, action) {
        var _a;
        try {
            const allowed = ((_a = this.resp.resourceInstances[resourceKey]) === null || _a === void 0 ? void 0 : _a.actions[action]) ==
                AuthorizeEffect.ALLOW;
            return allowed !== null && allowed !== void 0 ? allowed : false;
        }
        catch (e) {
            return false;
        }
    }
}
export class Cerbos {
    constructor({ hostname, logLevel, playgroundInstance, handleValidationErrors, }) {
        this.host = hostname;
        this.playgroundInstance = playgroundInstance;
        this.handleValidationErrors = handleValidationErrors;
        this.log = log.noConflict();
        this.log.setLevel(logLevel !== null && logLevel !== void 0 ? logLevel : "silent");
    }
    check(data) {
        return __awaiter(this, void 0, void 0, function* () {
            this.log.info("Cerbos.check", data);
            const payload = Object.assign(Object.assign({ requestId: uuidv4() }, data), { resource: Object.assign({ policyVersion: data.resource.policyVersion || "default" }, data.resource), principal: Object.assign({ policyVersion: data.principal.policyVersion || "default" }, data.principal) });
            this.log.debug("Cerbos.check Payload", payload);
            let headers = {
                "Content-Type": "application/json",
            };
            if (this.playgroundInstance) {
                headers = Object.assign(Object.assign({}, headers), { "Playground-Instance": this.playgroundInstance });
            }
            let resp;
            // Fetch Data
            try {
                const response = yield fetch(`${this.host}/api/check`, {
                    method: "post",
                    body: JSON.stringify(payload),
                    headers,
                });
                resp = yield response.json();
                this.log.info("Cerbos.check: Response", resp);
            }
            catch (e) {
                this.log.error("Cerbos.check Error", e);
                throw new AuthorizationError(`Could not connect to Cerbos PDP at ${this.host}`);
            }
            // Handle Validation Errors
            if (this.handleValidationErrors) {
                const validationErrors = resp.resourceInstances
                    ? Object.values(resp.resourceInstances)
                        .map((resource) => resource.validationErrors)
                        .flat()
                    : [];
                if (validationErrors.length > 0) {
                    if (this.handleValidationErrors === "error") {
                        throw new AuthorizationError(`Validation Error: ${JSON.stringify(validationErrors)}`);
                    }
                    else {
                        this.log.error("Cerbos.check: Validation Errors", validationErrors);
                    }
                }
            }
            return new CerbosResponseWrapper(resp);
        });
    }
}
